@@ -41,6 +41,7 @@ from pre_workbench import windows
 from pre_workbench.configs import getIcon, SettingsSection, SettingsField, icon_searchpaths
 from pre_workbench.datawidgets import DynamicDataWidget
 from pre_workbench.util import get_app_version, SimpleThread
+from pre_workbench.windows.content.objectwindow import ObjectWindow
 from pre_workbench.windows.dialogs.manageannotationsets import ManageAnnotationSetsDialog
 # noinspection PyUnresolvedReferences
 from pre_workbench.windows.dockwindows import FileBrowserWidget, MdiWindowListWidget, StructInfoTreeWidget, \
@@ -196,15 +197,15 @@ class WorkbenchMain(QMainWindow):
 		self.mapChildAction(self.reloadFileAct, "reloadFile")
 
 		# TODO
-		"""self.closeAct = QAction("Cl&ose", self,
+		self.closeAct = QAction("Cl&ose", self, shortcut='Ctrl+W',
 				statusTip="Close the active window",
-				triggered=self.mdiArea.closeActiveSubWindow)
+				triggered=self.closeActiveChildWindow)
 
-		self.closeAllAct = QAction("Close &All", self,
+		self.closeAllAct = QAction("Close &All", self, shortcut='Ctrl+Shift+W',
 				statusTip="Close all the windows",
-				triggered=self.mdiArea.closeAllSubWindows)
+				triggered=self.closeAllChildWindows)
 
-		self.nextAct = QAction("Ne&xt", self, shortcut=QKeySequence.NextChild,
+		"""self.nextAct = QAction("Ne&xt", self, shortcut=QKeySequence.NextChild,
 				statusTip="Move the focus to the next window",
 				triggered=self.mdiArea.activateNextSubWindow)
 
@@ -275,6 +276,8 @@ class WorkbenchMain(QMainWindow):
 			#a = toolWndMenu.addAction(name, lambda name=name: self.mdiArea.findDockWidget(name).toggleView(True))
 			toolWndMenu.addAction(self.mdiArea.findDockWidget(name).toggleViewAction())
 			toolWndToolbar.addAction(self.mdiArea.findDockWidget(name).toggleViewAction())
+		toolbarsMenu = viewMenu.addMenu('Tool Bars')
+		toolbarsMenu.addActions([mainToolbar.toggleViewAction(), toolWndToolbar.toggleViewAction()])
 		viewMenu.addSeparator()
 		a = QAction("Zoom In", self, shortcut='Ctrl++')
 		self.mapChildAction(a, "zoomIn")
@@ -415,18 +418,21 @@ class WorkbenchMain(QMainWindow):
 
 	def showAboutBox(self):
 		from pre_workbench import errorhandler
+		def link(href, prefix=""):
+			if href is None: return "None"
+			return f"<a href='{prefix}{href}'>{href}</a>"
 		showWidgetDlg(QTextBrowser(minimumWidth=520,minimumHeight=300,searchPaths=icon_searchpaths,openLinks=False,anchorClicked=navigateBrowser,
 							html="<img src='appicon.png' style='float: left; '><br><b>Protocol Reverse Engineering Workbench</b><br>"
-							"Version " + get_app_version() +"<br><br>"
+							f"Version {get_app_version()}<br><br>"
 							"Copyright (c) 2022 Mira Weller<br>"
 							"Licensed under the GNU General Public License Version 3<br style='clear:left'><br><hr>"
 							"<table>"
-							"<tr><td>Website: </td><td><a href='"+ consts.WEBSITE_URL + "'>"+ consts.WEBSITE_URL + "</a></td></tr>"
-							"<tr><td>Issue Tracker: </td><td><a href='"+ consts.ISSUE_TRACKER_URL + "'>"+ consts.ISSUE_TRACKER_URL + "</a></td></tr>"
+							f"<tr><td>Website: </td><td>{link(consts.WEBSITE_URL)}</td></tr>"
+							f"<tr><td>Issue Tracker: </td><td>{link(consts.ISSUE_TRACKER_URL)}</td></tr>"
 							"<tr><td>&nbsp;</td><td></td></tr>"
-							"<tr><td>Config Dir: </td><td><a href='file:" + configs.dirs.user_config_dir + "'>" + configs.dirs.user_config_dir + "</a></td></tr>"
-							"<tr><td>Log File: </td><td>" + errorhandler.logFile + "</td></tr>"
-							"<tr><td>Plugins Dir: </td><td><a href='file:" + APP().plugins_dir + "'>" + APP().plugins_dir + "</a></td></tr>" +
+							f"<tr><td>Config Dir: </td><td>{link(configs.dirs.user_config_dir, 'file:')}</td></tr>"
+							f"<tr><td>Log File: </td><td>{errorhandler.logFile}</td></tr>"
+							f"<tr><td>Plugins Dir: </td><td>{link(APP().plugins_dir, 'file:')}</td></tr>" +
 							("<tr><td>Loaded Plugins:</td><td>" + "<br>".join(APP().plugins.keys())+"</td></tr>" if len(APP().plugins) > 0 else "")
 							+ "</table>"),
 					  "About PRE Workbench", lambda: None, self, buttons=QDialogButtonBox.Close)
@@ -454,6 +460,7 @@ class WorkbenchMain(QMainWindow):
 			SettingsField("definition", "Start Grammar Definition", "text", {"listselectcallback":formatinfoSelect}),
 			SettingsField("only_types", "Only Export These Definitions", "text", {"listselectcallback":formatinfoSelect,"multiselect":","}),
 			SettingsField("dissector_table", "Dissector Table", "text", {}),
+			SettingsField("raise_not_implemented", "Raise NotImplementedErrors", "check", {"default":True}),
 			SettingsField("output_file", "Output File", "text", {"fileselect":"save","filter":"Lua Script (*.lua)"}),
 		], self.project.getValue("LastWiresharkExportOptions", {}), "Wireshark Dissector Export", self, min_width=550,
 		help_callback=lambda: navigateBrowser(WDGEN_HELP_URL))
@@ -461,7 +468,9 @@ class WorkbenchMain(QMainWindow):
 			from pre_workbench.wdgen.lua import generate_lua_dissector
 			self.project.setValue("LastWiresharkExportOptions", r)
 			with open(r['output_file'], 'w') as out:
-				generate_lua_dissector(r['definition'], r['only_types'], r['dissector_table'].split(','), self.project.formatInfoContainer, out)
+				generate_lua_dissector(r['definition'], r['only_types'], r['dissector_table'].split(','), self.project.formatInfoContainer, r['raise_not_implemented'], out)
+			with open(r['output_file'], 'r') as infile:
+				self.zoomWindow.setContents(infile.read())
 
 	def onMruClicked(self):
 		self.openFile(self.sender().data())
@@ -476,21 +485,19 @@ class WorkbenchMain(QMainWindow):
 
 	def _updateWindowMenu(self):
 		#TODO
-		return
 		self.windowMenu.clear()
 		self.windowMenu.addAction(self.closeAct)
 		self.windowMenu.addAction(self.closeAllAct)
 		self.windowMenu.addSeparator()
-		self.windowMenu.addAction(self.nextAct)
-		self.windowMenu.addAction(self.previousAct)
+		#self.windowMenu.addAction(self.nextAct)
+		#self.windowMenu.addAction(self.previousAct)
 
-		windows = self.mdiArea.dockWidgetsMap().values()
-		if len(windows) != 0: self.windowMenu.addSeparator()
+		windows = self.getChildWindows()
 
 		for i, window in enumerate(windows):
 			child = window.widget()
 
-			text = "%d %s" % (i + 1, child.windowTitle())
+			text = "%d %s" % (i + 1, window.windowTitle())
 			if i < 9:
 				text = '&' + text
 
@@ -500,6 +507,19 @@ class WorkbenchMain(QMainWindow):
 			action.triggered.connect(self.windowMapper.map)
 			self.windowMapper.setMapping(action, window)
 
+	def getChildWindows(self):
+		return [wnd for wnd in self.mdiArea.dockWidgetsMap().values() if
+				(wnd.features() & ads.CDockWidget.DockWidgetDeleteOnClose) != ads.CDockWidget.NoDockWidgetFeatures]
+
+	def closeActiveChildWindow(self):
+		activeSubWindow = self.mdiArea.focusedDockWidget()
+		if activeSubWindow:
+			activeSubWindow.closeDockWidget()
+
+	def closeAllChildWindows(self):
+		for wnd in self.getChildWindows():
+			wnd.closeDockWidget()
+
 	def _updateParserMenu(self):
 		self.loadAnnotationSetMenu.clear()
 		self.applyFormatInfoMenu.clear()
@@ -632,7 +652,7 @@ class WorkbenchMain(QMainWindow):
 
 		root,ext = os.path.splitext(FileName)
 		winType, _ = WindowTypes.find(fileExts=ext)
-		if winType is None: winType = HexFileWindow
+		if winType is None: winType = ObjectWindow
 
 		try:
 			wnd = winType(fileName=FileName)
